/*-----------------------------------------------------------------------------
	give_tf_weapon, or "GTFW" for short,
	Version 5.4.1
	By Yakibomb
*///---------------------------------------------------------------------------


/*-----------------------------------------------------------------------------
	PLEASE use this file  as reference!!
	It has settings and OnGameEvents that might be used by other scripts!!
*///---------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// How to Use this Script in Your Maps
//--> To execute this script, make sure to place these files in directory `/tf/scripts/vscripts/`
//--> Then to execute this script in a map, make an entity with "vscripts" keyvalue, and place `give_tf_weapon/_master.nut` as the parameter
//--> Alternatively, you can test it in any map through console by typing `script_execute give_tf_weapon/_master`
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
/*	-> Required Settings for your map: <-

	You need to add a few GTFW functions in certain Game Events for this to work properly.
	
	They are:
->	"post_inventory_application"
->	"player_death"

	Please read below for what to add.

*///---------------------------------------------------------------------------


	ClearGameEventCallbacks()	//Clears any pre-existing OnGameEvent listeners. WARNING: If you're using other events, be aware that this is here or just comment this function out.

// Player regenerate event
// GTFW uses this function for cleaning up any unused entities that would otherwise clutter and possibly crash the server if ent count gets too high.

function OnGameEvent_post_inventory_application(params)
{
	if (!("userid" in params)) return;
//Defines player handle as hPlayer
	local hPlayer = GetPlayerFromUserID(params.userid);

// REQUIRED ( Must be at the beginning! )
	hPlayer.GTFW_Cleanup();	// Anytime a weapon or wearable is created, it won't delete by itself!! This function will remove those entities for you.
	

// OPTIONAL
// - LOADOUTS
//	 This feature is for if you want to save and load player loadouts.
//	 Without this generally what happens is after you give a weapon, resupply will just delete it completely and not give it back.
//	 This fixes that!
	EntFireByHandle(hPlayer,"RunScriptCode","self.LoadLoadout()",0.0,hPlayer,hPlayer);
	
// - CUSTOM CLASS ARMS FOR FIRST-PERSON
//	 As it says on the tin. This one is only if you want to restore custom class arms after touching resupply.
//	 You can set custom arms with function hPlayer.SetCustomClassArms(my_model_path, true)
//	  If you set param 2 to `true`, it'll save the arms in memory to be loaded by the below function.
//	  Otherwise, setting to `false` or just not including it won't save it in memory. It'll only last until GTFW_Cleanup() removes it.
	EntFireByHandle(hPlayer,"RunScriptCode","self.GTFW_RestoreCustomClassArms()",0.0,hPlayer,hPlayer);
}

// On player death event
// When a player dies, this updates their ragdoll to the class they're using.
// It's a fix the new feature in 5.4.0--Weapons that are unintended for the class using them update the animations to the intended class
// If you don't use that feature, then you don't need to include this event.
function OnGameEvent_player_death(params)	//Defines post_inventory_application event listener
{
	if (!("userid" in params)) return;

//Defines player handle as hPlayer
	local hPlayer = GetPlayerFromUserID(params.userid);
// Updates ragdoll to the player's actual class.
	GTFW.SetUseClassAnimations_OFF(hPlayer)
}


	__CollectGameEventCallbacks(this)	// Adds all the game events defined before this and makes them work. Yeah!


//-----------------------------------------------------------------------------
//	Weapon Config for GiveWeapon() and RegisterCustomWeapon()
//-----------------------------------------------------------------------------
// These are properties that are used by GTFW.
// You can preset the defaults for every weapon spawned by GiveWeapon(), or on a per-weapon basis.
// For per-weapon, use parameter 2 in GiveWeapon() and add the bits you want, or simply make a custom weapon with a different preset.

// `Defaults` preset for all weapons is below this list.

// Here are all the properties you can use:

::DeleteAndReplace <-	(1 << 0)	// Deletes the weapon that matches the slot of the new weapon, then adds the new weapon. Not compatible with KeepIDX bit. NOTE: Cannot switch to another weapon in the same slot, unless using "hud_fastswitch 0".
::KeepIDX <-			(1 << 1)	// Only updates the Item Definition Index of the given weapon. Not compatible with DeleteAndReplace bit. Added for MvM to allow for custom weapons to be upgradeable.
::AutoSwitch <-			(1 << 2)	// Forcefully switches to the weapon if obtained.
::WipeAttributes <-		(1 << 3)	// Clears original attributes present on the weapon.
::ForceCustom <-		(1 << 4)	// Forces the weapon to be "custom". "custom" in GTFW terms means different things, but here it means it sets netprop "ItemIDHigh" bit 6, for coding convenience. (The other "custom" means unique ItemDefinitionIndex. Don't worry about it right now)
::ForceNotCustom <-		(1 << 5)	//  Makes sure to unset "ItemIDHigh" bit 6.
::AnnounceToChat <- 	(1 << 6)	// Announces the weapon in chat for all to see what you got!
::Save <- 				(1 << 7)	// Saves the weapon, allowing players to retrieve it as long as function "handle.LoadLoadout()" is used.
::AutoRegister <- 		(1 << 8)	// This automatically registers this as a custom weapon, if it wasn't already registered before. It might suck up all the unique ItemDefinitionIndexes, so use with caution, or don't use this setting at all.
::FixClassAnims <- 		(1 << 9)	// Player model switches to class animations of weapon's intended player class. No disgression--If it the weapon is unintended for that class, it will switch.
::FixClassAnimsRef <-	(1 << 10)	//  Same as FixClassAnims, but only for when player goes into reference pose.
::FixClassAnimsITEM <-	(1 << 11)	//  Same as FixClassAnims, but only for unique ITEM animations (ITEM1, ITEM2, ITEM3, ITEM4).

//-----------------------------------------------------------------------------
// This is the Defaults preset for all weapons. Adjust it how you like.
//-----------------------------------------------------------------------------
//Defaults: DeleteAndReplace|AutoSwitch|Save|FixClassAnimsRef|FixClassAnimsITEM
//To remove all, write: 0

::Defaults <- DeleteAndReplace|AutoSwitch|Save|FixClassAnimsRef|FixClassAnimsITEM


//-----------------------------------------------------------------------------
//	VScript CVars
//-----------------------------------------------------------------------------
Convars.SetValue("tf_dropped_weapon_lifetime", 0)	//disables dropped weapons because they're buggy with this script
::CVAR_GTFW_DEBUG_MODE <- true		// Sends error messages to everyone. False by default.

::CVAR_USE_VIEWMODEL_FIX <- true				// Automatically fixes any and all viewmodel arms to match the class you're playing as. True by default.
::CVAR_DELETEWEAPON_AUTO_SWITCH <- true			// Automatically switches weapon to another if deleting a weapon. True by default.
::CVAR_DISABLEWEAPON_AUTO_SWITCH <- true		// Automatically switches weapon to another if disabling a weapon. True by default.
::CVAR_ENABLEWEAPON_AUTO_SWITCH <- false		// Automatically switches weapon to another if re-enabling a weapon. False by default.

const GLOBAL_WEAPON_COUNT = 7	//How many loops the script makes to check for unused slots

//-----------------------------------------------------------------------------
//	Included Scripts
//-----------------------------------------------------------------------------
// Only runs the main code files once to improve server performance.

//If TF_CUSTOM_WEAPONS_REGISTRY is already defined, stop running the script. Otherwise, continue to execute the main code files.
//if ( "GTFW" in getroottable() ) return;
::GTFW <- {};

IncludeScript("give_tf_weapon/extra/text_listener.nut")		//Optional chat command "!give"
IncludeScript("give_tf_weapon/libs/netpropperf.nut")	//according to ficool2, reduces NetProps runtime by some-20%!
IncludeScript("give_tf_weapon/code/__exec.nut")			// Executes all functions that this script uses!
IncludeScript("give_tf_weapon/custom_weapons/_all.nut")	// Executes all custom weapons written in the file!