//-----------------------------------------------------------------------------
//	Purpose: Gives a weapon to the player.
//			 This script's bread 'n' butter.
//-----------------------------------------------------------------------------

::CTFPlayer.GiveWeapon <- function(weapon,ItemDefinitionIndex_Override=null,gw_props=Defaults,PostWepFix_Override=null,stats_function_Override=null)
{
// Starts with GetWeaponTable(weapon).
// This searches for the weapon in the database, and returns an assembled table with all the properties we need to make this weapon!
	local table;
// if we just put a classname in "weapon", pick the first "if" statement. Else, use the second statement.
	local weapon_String_Classname_Type = false;
	local weapon_tolower = type ( weapon ) == "string" ? weapon.tolower() : "";
	if ( weapon_tolower && startswith(weapon_tolower, "tf_wea") ) {
		if ( weapon_tolower.len() < 13 ) return GTFW.DevPrint(1,"GiveWeapon", "Function failed. Incomplete string \x22"+weapon+"\x22. Returning null.");
		
		weapon_String_Classname_Type = startswith(weapon_tolower, "tf_weapon_") ? "tf_weapon_" : "tf_wearable_";
		weapon_tolower = startswith(weapon_tolower, "tf_weapon_") ? weapon_tolower.slice(10) : weapon_tolower.slice(12);
		table = GetWeaponTableNoPlayer(weapon_tolower);
	}
	else { table = this.GetWeaponTable(weapon); }
	
// Error if there is no valid item found
	if ( table == null ) {
		GTFW.DevPrint(1,"GiveWeapon", "Function failed at finding item \x22"+weapon+"\x22. Returning null.");
		return null;
	}
	
// Fixes up the weapon if override present
	if ( type( PostWepFix_Override ) == "function" ) {
		PostWepFix_Override(table, this);
	}
	
// Define variables
	local player = this;
	
// Establish all variables from the table inside this script
	local WepC				= table.classname;
	local PlayerTFClass		= table.tf_class;
	local ItemID			= table.itemID;
	local Slot				= table.slot;
	local AmmoType			= table.ammoType;
	local ClipSize			= table.clipSize;
	local MaxClipSize		= table.clipSizeMax;
	local AmmoReserve		= table.reserve;
	local MaxAmmoReserve	= table.reserveMax;
	local Extra_Wearable	= table.wearable;
	local Extra_Wearable_VM	= table.wearable_vm;
	local AnnounceQuality	= table.announce_quality;
	local AnnouncePrefix	= table.announce_prefix;
	local AnnounceHasString	= table.announce_has_string;
	
	local worldModel		= table.worldModel;
	local viewModel		 	= table.viewModel;
	local stats_function 	= table.func;
	local classarms		 	= table.classArms;
	
	if ( ItemDefinitionIndex_Override ) {
		local overrides	= GetWeaponTableNoPlayer(ItemDefinitionIndex_Override)
		ItemID			= overrides.itemID;
		PlayerTFClass	= overrides.tf_class;
	}

// Purpose: Create weapon by classname
// If you wanted to create a weapon by classname + Item Definition Index, this is the way.
	if ( weapon_String_Classname_Type ) { // set to `true` at beginning of script
		WepC = weapon_String_Classname_Type + weapon_tolower;	//adds "tf_weapon_" or "tf_wearable_" back
	}

	
// Purpose: Extra properties for the weapon.
//	It utilizes a set of bits. This works exactly how the most entities with `effects` keyvalue can hold so many options inside one number!
	if ( type ( gw_props ) != "integer" ) { gw_props = Defaults; }
	else { gw_props = gw_props | table.gw_props; }
	
	local bit_DeleteAndReplace	= gw_props & DeleteAndReplace ? true : false;
	local bit_KeepIDX			= gw_props & KeepIDX ? true : false;
	local bit_AutoSwitch		= gw_props & AutoSwitch ? true : false;
	local bit_WipeAttributes	= gw_props & WipeAttributes ? true : false;
	local bit_ForceCustom		= gw_props & ForceCustom ? true : false;
	local bit_ForceNotCustom	= gw_props & ForceNotCustom ? true : false;
	local bit_AnnounceToChat	= gw_props & AnnounceToChat ? true : false;
	local bit_SaveWeapon		= gw_props & Save ? true : false;
	local bit_AutoRegister		= gw_props & AutoRegister ? true : false;

	
// Purpose: Weapon Creation
// It finds a slot to replace, deletes that weapon in slot, then creates either a passive item (i.e. booties, etc) or a active weapon.

	local GivenWeapon;

// This thing is might be huge, but the bulk of it is finding and deleting passive items. (also, active ones)
	if (  bit_KeepIDX == false ) {
		//If ReplaceWeapon is set in bitmasks, it finds the weapon in the slot the new weapon would be in, then deletes it.
		if ( bit_DeleteAndReplace ) {
			for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++) {
					
				local wep = GetPropEntityArray(player, "m_hMyWeapons", i);
				
				if ( wep == null ) continue
					if ( wep.GetSlot() == Slot ) {
						SetPropEntityArray(player, "m_hMyWeapons", null, i);
						KillWepAll(wep);
						break
					}
			}
			
	// Finds passive weapons like Gunboats, Targe, etc.
			local item = player.GetPassiveWeaponBySlot(Slot);
			if ( item ) {

				if ( item.GetClassname() == "tf_wearable_demoshield" )
					SetPropInt(player, "m_Shared.m_bShieldEquipped", 0);
				
				KillWepAll(item);
			}
			GTFW.DevPrint(0,"GiveWeapon", "Removed item from slot "+Slot+"!");
		}
		
	// Purpose: This creates the new items
	// Passive items: If the classname matches any of these under SearchPassiveItems (list in tables.nut), it creates a wearable. The "CreateCustomWearable()" function does the rest.
	// Active items: else, it creates the new active weapon using the "AddWeapon" function.
	
	// Purpose: Creates a passive item	
		if ( SearchPassiveItems.find(WepC) ) {
			if ( ItemID < 65535						//if not custom. If bit 16 or over is set, it goes over this number
			 || WepC == "tf_wearable_demoshield" )	//if a demoshield
				GivenWeapon = player.CreateCustomWearable(WepC, null, ItemID);
			else 
				GivenWeapon = player.CreateCustomWearable(WepC, worldModel, ItemID);
			
			//Demoshield fix
			if ( WepC == "tf_wearable_demoshield" )
				SetPropInt(player, "m_Shared.m_bShieldEquipped", 1);
		}
		else {
	// Purpose: Creates an active weapon
			for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++) {
				
				local wep = GetPropEntityArray(player, "m_hMyWeapons", i);
				
				if ( wep ) continue
					GivenWeapon = player.AddWeapon(WepC, ItemID, i);
					break;
			}
		}
	}
	
// If "bit_keepIDX" set to true, it will only change the item definition index of the weapon.
	else {
		GivenWeapon = player.ReturnWeaponBySlot(Slot);
		if ( ItemID >= 0 ) {
			SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", ItemID );
		}
		else {
			SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", GetItemID(GivenWeapon) );
		}
		GTFW.DevPrint(0,"GiveWeapon", "bit_keepIDX = true. \x22"+weapon+"\x22 updated to IDX "+GetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")+"!");
	}
	
// Purpose: seems to throw an error only if it didn't create a weapon,
// that can only happen if weps in `m_hMyWeapons` exceeds GLOBAL_WEAPON_COUNT
	if ( GivenWeapon == null ) {
		GTFW.DevPrint(1,"GiveWeapon", "Function failed at giving weapon \x22"+weapon+"\x22. Returning null.");
		return null;
	}
	
	
// Purpose: Meant for CTFPlayer.GetPassiveWeaponBySlot()
// Should fix passive weapons not being found in slots >7.
	if ( GivenWeapon.GetClassname() == "tf_wearable" )
		GivenWeapon.KeyValueFromString("classname","tf_wearable_vs");
	
// If our weapon has a custom model or custom arms, makes it custom.

	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope()
	local VM_Scope = main_viewmodel.GetScriptScope();
	
	if ( !bit_ForceNotCustom
	 && (ItemID & (1 << 16))
	 || worldModel || viewModel || classarms
	 || bit_ForceCustom
	 || ItemDefinitionIndex_Override
	 || ( player.GetPlayerClass() != PlayerTFClass && PlayerTFClass != 0 ) 
	 || "classarms" in VM_Scope )
	{
		IS_CUSTOM(GivenWeapon, true);	//marks our weapon as custom (which means "m_iItemIDHigh" bit 6 set)
	}
	else if ( bit_ForceNotCustom ) {
		IS_CUSTOM(GivenWeapon, false)	//removes the custom weapon bits
	}

	
// Modifies ammo if ammo types don't match
	if ( HasProp(GivenWeapon, "LocalWeaponData.m_iPrimaryAmmoType") && AmmoType != GivenWeapon.GetPrimaryAmmoType() ) {
		SetPropInt(GivenWeapon, "LocalWeaponData.m_iPrimaryAmmoType", AmmoType);
	}
	
// Purpose: Custom stats function
// Forced overrides parameters...
	if ( stats_function_Override ) {
		stats_function = stats_function_Override;
	}
// Runs custom weapon function if it exists
	if ( type( stats_function ) == "function" ) {
		stats_function(GivenWeapon, player);
		GivenWeapon.ReapplyProvision();
	}
	
// If our weapon does not use original stats, clear them.
	if ( bit_WipeAttributes ) {
		SetPropInt(GivenWeapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1);	// Removes all pre-existing stats of the weapon.
	}
	
// Purpose: Extra wearable ( if applicable )
	if ( Extra_Wearable ) {
		player.CreateCustomWearable("tf_wearable", Extra_Wearable, null, GivenWeapon);
	}
	
// Purpose: Extra wearable for viewmodels ( if applicable )
	if ( Extra_Wearable_VM ) {
		player.CreateCustomWearable("tf_wearable_vm", Extra_Wearable_VM, null, GivenWeapon);
	}

// Purpose: For updates weapon models
// for UpdateCustomWeapon()
	if ( HasProp(GivenWeapon, "m_iWorldModelIndex") && worldModel == null && viewModel == null ) {
		worldModel = GetPropInt(GivenWeapon, "m_iWorldModelIndex");
		viewModel = GetPropInt(GivenWeapon, "m_iWorldModelIndex");
	}
	else if ( worldModel == null && viewModel == null ) {
		worldModel = GetPropInt(GivenWeapon, "m_nModelIndex");
		viewModel = GetPropInt(GivenWeapon, "m_nModelIndex");
	}
	if ( worldModel == null && viewModel) {
		worldModel = viewModel;
	}
	else if ( worldModel && viewModel == null) {
		viewModel = worldModel;
	}

// Write GTFW table to weapon scope
	GivenWeapon.ValidateScriptScope();
	local wep_scope = GivenWeapon.GetScriptScope();
	table.WriteTableToScope(wep_scope);
// This tells the game it came here and only here
	wep_scope.from_GiveWeapon <- true;
	
// Purpose: Finalize weapon.
// Adds AmmoFix wearable (to fix ammo), updates class arms/viewmodels, updates world weapon models, and adds viewmodel's Think script
	GTFW.UpdateAmmoFix(player, GivenWeapon,PlayerTFClass,AmmoType,ClipSize,MaxClipSize,AmmoReserve,MaxAmmoReserve);
	GTFW.UpdateArms(player, GivenWeapon, null);
	GTFW.UpdateCustomWeapon(player, GivenWeapon, worldModel, viewModel);
	GTFW.AddThinkToViewModel(player, GivenWeapon);
	
	
// Purpose: Switches to another weapon if active one was deleted
	if ( bit_AutoSwitch )
		player.SwitchToBest(GivenWeapon);
	
	
// Purpose: Announces weapon in text chat that it has been found
	if ( bit_AnnounceToChat ) player.AnnounceWeapon(GetItemID(GivenWeapon),AnnounceQuality,AnnouncePrefix,AnnounceHasString);
		
// Purpose: Saves weapon so resupply or dying won't remove the weapon
	if ( bit_SaveWeapon ) player.SaveLoadout(weapon);
	
// Purpose: Automatically registers your given weapon as custom
// *Only use when you need it.* There can only be 63 custom weapons using the same item definition index.
	if ( bit_AutoRegister )
		RegisterCustomWeapon(UniqueString(), weapon, PostWepFix_Override,stats_function_Override);
	

// Purpose: Echoes to console who received what weapon
	if ( type( weapon ) == "integer" )
		weapon = table.name;
	GTFW.DevPrint(0,"GiveWeapon", "Gave "+GTFW.GetNetName(player)+" item \x22"+weapon+"\x22!");

	return GivenWeapon
}


//-----------------------------------------------------------------------------
//Base for making weapons. Base weapon netprops compiled by ficool2.
//-----------------------------------------------------------------------------
::CTFPlayer.AddWeapon <- function(classname, itemindex, slot)
{
	GTFW.DevPrint(0,"AddWeapon",format("%s %i %i",classname, itemindex, slot))
	local ply = this;
	local weapon = SpawnEntityFromTable(classname, {
		origin = ply.GetOrigin(),
		angles = ply.GetAbsAngles(),
		effects = 129,
		TeamNum = ply.GetTeam(),
		CollisionGroup = 11,
		ltime = Time(),
	});
	
	SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex);
	SetPropInt(weapon, "m_AttributeManager.m_Item.m_iEntityLevel", 0);
	SetPropBool(weapon, "m_AttributeManager.m_Item.m_bInitialized", true);
	
	SetPropBool(weapon, "m_bClientSideAnimation", true);
	SetPropBool(weapon, "m_bClientSideFrameReset", true);
	
	SetPropBool(weapon, "m_bValidatedAttachedEntity", true);
	//SetPropInt(weapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1);	// Removes all pre-existing stats of the weapon.
	SetPropInt(weapon, "m_AttributeManager.m_iReapplyProvisionParity", 1);
	
	SetPropEntity(weapon, "m_hOwner", ply);
	weapon.SetOwner(ply);
	
// Seems without this, we can't hit Engineer objects!
	local solidFlags = GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags | Constants.FSolid.FSOLID_NOT_SOLID);
	
	solidFlags = GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags & ~(Constants.FSolid.FSOLID_TRIGGER));
	

	Entities.DispatchSpawn(weapon)	//Dispatches weapon into the world
	weapon.ReapplyProvision()	// then applies any body attributes back onto the player.


	SetPropEntityArray(ply, "m_hMyWeapons", weapon, slot);

	DoEntFire("!self", "SetParent", "!activator", 0, ply, weapon);

	itemindex = GetItemID(itemindex,ForceNotCustom)	//clears custom bits

//Engy Toolbox
	if ( classname == "tf_weapon_builder" )	//IDX 28
	{
		GTFW.DevPrint(0,"AddWeapon", format("Classname %s is updating ",classname));
		
		SetPropInt(weapon, "BuilderLocalData.m_iObjectType", 0)
		SetPropInt(weapon, "m_iSubType", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.000", 1)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.001", 1)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.002", 1)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.003", 0)
	}
//Sappers
	if ( classname == "tf_weapon_sapper" ) // IDX 735 and 736 are classname tf_weapon_builder in items_game.txt; opted to change them to tf_weapon_sapper in tables.nut for QoL
	{
		GTFW.DevPrint(0,"AddWeapon", format("Classname %s %i is updating ",classname,itemindex));
		
		SetPropInt(weapon, "BuilderLocalData.m_iObjectType", 3)
		SetPropInt(weapon, "m_iSubType", 3)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.000", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.001", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.002", 0)
		SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.003", 1)
	}
	
	if ( classname.find("lunchbox")
	  || classname.find("jar") 
	  || classname == "tf_weapon_bat_wood"
	  || classname == "tf_weapon_cleaver" ) {
		SetPropFloat(weapon, "LocalActiveTFWeaponData.m_flEffectBarRegenTime", Time() + 0.03);
		if ( !classname.find("_gas") ) { 
			DoEntFire("!self","RunScriptCode","SetPropIntArray(self, `m_iAmmo`, 1, 5)",0.0,ply,ply)
			DoEntFire("!self","RunScriptCode","SetPropIntArray(self, `m_iAmmo`, 1, 4)",0.0,ply,ply)
		}
		else {
			DoEntFire("!self","RunScriptCode","SetPropIntArray(self, `m_iAmmo`, 0, 5)",0.0,ply,ply)
			DoEntFire("!self","RunScriptCode","SetPropIntArray(self, `m_iAmmo`, 0, 4)",0.0,ply,ply)
		}
	}

	GTFW.DevPrint(0,"AddWeapon", "Initialized \x22"+weapon+"\x22.")
	return weapon
}

//-----------------------------------------------------------------------------
// Purpose: Fixes ammo.
// There is an invisible tf_wearable that tracks ammo for custom weapons, and DisableWeapon/EnableWeapon
// Same goes for metal ammotype weapons like Widowmaker, Wrenches, PDAs etc
//-----------------------------------------------------------------------------
::GTFW.UpdateAmmoFix <- function(player,weapon,WepTFClass,AmmoType,ClipSize,MaxClipSize,AmmoReserve,MaxAmmoReserve)
{
// throw an error if the weapon doesn't use ammo
	if ( !HasProp(weapon, "LocalWeaponData.m_iPrimaryAmmoType") ) {
		GTFW.DevPrint(1,"UpdateAmmoFix", weapon+" is not compatible with ammo fix!");
		return weapon;
	}

// Define variables
	local AmmoFix;
	local aThreshold = 0;
	local aPerClass = 0;
	local FinalAmmoReserve;
	local playerClass = player.GetPlayerClass();
	
	if ( AmmoType > TF_AMMO.NONE && MaxAmmoReserve >= 0 ) {
		
	//grabs handle for AmmoFix (a tf_wearable)
		AmmoFix = GTFW.GetAmmoFix(player);
		
	// Purpose: Updates primary ammo reserve
		if ( AmmoType == TF_AMMO.PRIMARY ) {
			aPerClass = MaxAmmoReserve == -1 ? TF_AMMO_PER_CLASS_PRIMARY[WepTFClass] : MaxAmmoReserve;
			aThreshold = TF_AMMO_PER_CLASS_PRIMARY[playerClass];
			AmmoFix.AddAttribute("hidden primary max ammo bonus", aPerClass.tofloat() / aThreshold.tofloat(), -1);
		}
	// Purpose: Updates secondary ammo reserve
		else if ( AmmoType == TF_AMMO.SECONDARY ) {
			aPerClass = MaxAmmoReserve == -1 ? TF_AMMO_PER_CLASS_SECONDARY[WepTFClass] : MaxAmmoReserve;
			aThreshold = TF_AMMO_PER_CLASS_SECONDARY[playerClass];
			weapon.AddAttribute("hidden secondary max ammo penalty", aPerClass.tofloat() / aThreshold.tofloat(), -1);
		}
	// Purpose: Updates GRENADES1 ammo reserve
		else if ( AmmoType == TF_AMMO.GRENADES1 ) {
			aThreshold = 1;
			AmmoFix.AddAttribute("maxammo grenades1 increased", (MaxAmmoReserve.tofloat() / aThreshold.tofloat()), -1);
		}
	// Purpose: Updates metal capacity
		else if ( AmmoType == TF_AMMO.METAL ) {
			aThreshold = 200;
			AmmoFix.AddAttribute("maxammo metal increased", (MaxAmmoReserve.tofloat() / aThreshold.tofloat()), -1);
		}
		
	// Purpose: Applies all attributes to player
		AmmoFix.ReapplyProvision();
	}
	
	FinalAmmoReserve = MaxAmmoReserve;
// Purpose: Sets ammo reserve, but only if reserve is different than reserveMax
	if ( MaxAmmoReserve != AmmoReserve )
		FinalAmmoReserve = AmmoReserve.tofloat();
		
// Purpose: Sets ammo reserve
	if ( AmmoType == TF_AMMO.METAL && EntityOutputs.HasAction(AmmoFix, "OnUser3") == false && playerClass != Constants.ETFClass.TF_CLASS_ENGINEER ) {
		EntityOutputs.AddOutput(AmmoFix, "OnUser3", "", "", "", 0.0, -1);	//used to flag for Metal ammo so we don't update twice
		SetPropIntArray(player, "m_iAmmo", FinalAmmoReserve, TF_AMMO.METAL);
	}
	else {
		SetPropIntArray(player, "m_iAmmo", FinalAmmoReserve, AmmoType);
	}
	
	if ( AmmoType == TF_AMMO.METAL && EntityOutputs.HasAction(AmmoFix, "OnUser3") == false && playerClass != Constants.ETFClass.TF_CLASS_ENGINEER ) {
		EntityOutputs.AddOutput(AmmoFix, "OnUser3", "", "", "", 0.0, -1);	//used to flag for Metal ammo so we don't update twice
		SetPropIntArray(player, "m_iAmmo", AmmoReserve, TF_AMMO.METAL);
	}
	else {
		SetPropIntArray(player, "m_iAmmo", AmmoReserve, AmmoType);
	}
	// Purpose: Sets ammo clip
	// It seems like having both helps account for weapons that don't use the atomic upgrade, but atomic seems to supersede the %clipsize penalty for what ever reason
	if ( MaxClipSize >= 0 ) {
		if ( (MaxClipSize.tofloat() / weapon.GetMaxClip1().tofloat()) != 1 ) {
			weapon.AddAttribute("clip size upgrade atomic", 0 - (weapon.GetMaxClip1() - MaxClipSize), -1);
			weapon.AddAttribute("clip size penalty HIDDEN", MaxClipSize.tofloat() / weapon.GetMaxClip1().tofloat(), -1);
		}
		weapon.SetClip1(MaxClipSize);
	}
	if ( MaxClipSize != ClipSize ) {
		weapon.SetClip1(ClipSize);
	}
	
	return weapon
}

//-----------------------------------------------------------------------------
// Purpose: Creates an invisible tf_wearable with attributes that adjust ammo reserve
// First checks if it exists in player scope, if not, creates the invisible tf_wearable.
//-----------------------------------------------------------------------------
::GTFW.GetAmmoFix <- function(player=null) {
	if ( player && player.GetClassname() != "player" ) {
		GTFW.DevPrint(1,"GTFW.GetAmmoFix", "Function failed. Param 1 not a player. Returning null.");
		return null;
	}
	
	local AmmoFix;
	player.ValidateScriptScope()
	
	local plyScope = player.GetScriptScope();
	local ammofix;

	if ( "AMMOFIX" in plyScope ) { 
		ammofix = plyScope["AMMOFIX"];
		
		if ( ammofix && ammofix.IsValid() ) return ammofix;
		else {
			delete plyScope["AMMOFIX"];
		}
	}
	
	ammofix = player.CreateCustomWearable("tf_wearable");
	plyScope["AMMOFIX"] <- ammofix;

	GTFW.DevPrint(0,"CreateAmmoFix", "Created " + ammofix + " for " + GTFW.GetNetName(player));
	return ammofix;
}


//-----------------------------------------------------------------------------
// Purpose: Used to update viewmodel arms animations in GiveWeapon()
//-----------------------------------------------------------------------------
::GTFW.UpdateArms <- function(player, givenwep=null, force_class_arms=null)
{
// throw error if VM fix not enabled
	if ( !CVAR_USE_VIEWMODEL_FIX ) {
		GTFW.DevPrint(0,"UpdateArms", "CVAR_USE_VIEWMODEL_FIX = false. UpdateArms cancelled.")
		return null
	}
// throw error if weapon not compatible with class arms
// (tf_wearable and other passive items don't have "m_iPrimaryAmmoType" netprop)
	if ( !HasProp(givenwep, "LocalWeaponData.m_iPrimaryAmmoType") ) {
		GTFW.DevPrint(1,"UpdateArms", givenwep+" is not a compatible weapon!")
		return givenwep
	}
	
// Purpose: This is the special VM fix.
// it creates class arms that bonemerge with VM

// Adds a class arms model if nothing has parented to the main_viewmodel
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope()
	local VM_Scope = main_viewmodel.GetScriptScope();
	
	if ( "classarms" in VM_Scope ) {
		local arms = VM_Scope["classarms"]
		if ( !arms || !arms.IsValid() )
			delete VM_Scope["classarms"];
	}
	if ( !( "classarms" in VM_Scope ) ) {
		local arms;
		if ( player.HasGunslinger() && player.GetPlayerClass() == 9 )
			arms = GTFW_MODEL_ARMS[10];
		else 
			arms = GTFW_MODEL_ARMS[player.GetPlayerClass()];
		local classarms_model = player.CreateCustomWearable("tf_wearable_vm", arms, null, main_viewmodel);

		GTFW.DevPrint(0,"UpdateArms", "Created arms " + classarms_model);
		
	// Purpose: Puts bonemerged class arms model in main_viewmodel scope for easy tracking
		VM_Scope["classarms"] <- classarms_model;
		
	}
	
// Purpose: Writes to the scope above, which keep the class arms the weapon needs to update to when switched to.
	
	GTFW.WriteTableToWeaponScope_EntireLoadout(player);
	local loadout = player.GetAllWeapons();
	local tf_playerclass = player.GetPlayerClass();
	
	foreach (k, wep in loadout ) {
		if ( wep != givenwep ) continue
		local wepScope = wep.GetScriptScope();
		local model;
		local classn = wepScope.classname;
		local wepID_Clean = GetItemID(wep,ForceNotCustom);	//clears custom bits to allow us to use custom watches/PDAs
			
		if ( force_class_arms )
			model = force_class_arms;
		else if ( classn == "tf_weapon_pda_spy" )
			model = "models/weapons/v_models/v_pda_spy.mdl";
		else if ( classn == "tf_weapon_invis" && wepID_Clean == 30 )
			model = "models/weapons/v_models/v_watch_spy.mdl";
		else if ( classn == "tf_weapon_invis" && wepID_Clean == 59 )
			model = "models/weapons/v_models/v_watch_pocket_spy.mdl";
		else if ( classn == "tf_weapon_invis" && wepID_Clean == 60 )
			model = "models/weapons/v_models/v_watch_leather_spy.mdl";
		else if ( classn == "tf_weapon_invis" && wepID_Clean == 297 )
			model = "models/weapons/v_models/v_ttg_watch_spy.mdl";
		else if ( classn == "tf_weapon_invis" && wepID_Clean == 947 )
			model = "models/workshop_partner/weapons/v_models/v_hm_watch/v_hm_watch.mdl";
		else if ( classn == "tf_weapon_robot_arm" )
			model = GTFW_MODEL_ARMS[10];
		else if ( wepScope.classArms )
			model = wepScope.classArms;
		else if ( wepScope.tf_class == 0 || tf_playerclass == wepScope.tf_class )
			model = GTFW_MODEL_ARMS[tf_playerclass];
		else
			model = GTFW_MODEL_ARMS[wepScope.tf_class];
		
		PrecacheModel( model );
		local modelIndex = GetModelIndex( model );
		wep.SetModelSimple(model);
		wep.SetCustomViewModelModelIndex(modelIndex);
		NetProps.SetPropInt(wep, "m_iViewModelIndex", modelIndex);
		
		GTFW.DevPrint(0,"UpdateArms", "Updated weapon " + wep + " with model " + model);

		break;
	}
	return
}/*		local wepID = GetItemID(wep);
		local wepID_Clean = GetItemID(wep,ForceNotCustom);	//clears custom bits to allow us to use custom watches/PDAs
		local table = player.GetWeaponTable(wepID);

		if ( table && wep == givenwep || force_class_arms ) {
			local model;
			
			if ( force_class_arms )
				model = force_class_arms;
			else if ( table.classname == "tf_weapon_pda_spy" )
				model = "models/weapons/v_models/v_pda_spy.mdl";
			else if ( table.classname == "tf_weapon_invis" && wepID_Clean == 30 )
				model = "models/weapons/v_models/v_watch_spy.mdl";
			else if ( table.classname == "tf_weapon_invis" && wepID_Clean == 59 )
				model = "models/weapons/v_models/v_watch_pocket_spy.mdl";
			else if ( table.classname == "tf_weapon_invis" && wepID_Clean == 60 )
				model = "models/weapons/v_models/v_watch_leather_spy.mdl";
			else if ( table.classname == "tf_weapon_invis" && wepID_Clean == 297 )
				model = "models/weapons/v_models/v_ttg_watch_spy.mdl";
			else if ( table.classname == "tf_weapon_invis" && wepID_Clean == 947 )
				model = "models/workshop_partner/weapons/v_models/v_hm_watch/v_hm_watch.mdl";
			else if ( table.classname == "tf_weapon_robot_arm" )
				model = GTFW_MODEL_ARMS[10];
			else if ( table.classArms )
				model = table.classArms;
			else if ( table.tf_class == 0 || player.GetPlayerClass() == table.tf_class )
				model = GTFW_MODEL_ARMS[player.GetPlayerClass()];
			else
				model = GTFW_MODEL_ARMS[table.tf_class];
			
			PrecacheModel( model );
			local modelIndex = GetModelIndex( model );;
			wep.SetModelSimple(model);
			wep.SetCustomViewModelModelIndex(modelIndex);
			NetProps.SetPropInt(wep, "m_iViewModelIndex", modelIndex);
			
			GTFW.DevPrint(0,"UpdateArms", "Updated weapon " + wep + " with model " + model);

			break;
		}
	}
}*/


//-----------------------------------------------------------------------------
//	Purpose: Updates all weapon models for custom/unintended for tfclass weapons
//	This function works with AddThinkToViewModel() to make weapons visible/invisible as needed.
//   Things like weapon switching, taunting, etc, will make the replaced weapon invisible and make the vscript custom weapon visible.
//-----------------------------------------------------------------------------
 
::GTFW.UpdateCustomWeapon <- function(player, weapon, worldModel, viewModel)
{
// Throw error if param player isn't valid
	if ( type( player ) != "instance" || ( player && player.GetClassname() != "player" ) ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 1 is not a player. Returning null.");
		return null;
	}
	if ( type( weapon ) != "instance" ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 2 is not a weapon. Returning null.");
		return null;
	}
	if ( type( worldModel ) != "integer" && type( worldModel ) != "string" ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 3 is not an int or string. Returning null.");
		return null;
	}
	if ( type( viewModel ) != "integer" && type( viewModel ) != "string" ) {
		GTFW.DevPrint(1,"UpdateCustomWeapon", "Function failed. Parameter 4 is not an int or string. Returning null.");
		return null;
	}
	
	
// Define variables
	local WepC = weapon.GetClassname();
	local Is_DemoShield = WepC == "tf_wearable_demoshield";
	
// Purpose: Only update if the weapon is custom, or a demoshield
// Deletes custom weapon models for ViewModel, then recreates them
	if ( IS_CUSTOM(weapon)	// If weapon is custom
	 || Is_DemoShield )		// or a demoshield
	{
		GTFW.DevPrint(0,"UpdateCustomWeapon", "Updating \x22"+weapon+"\x22.");
		
		local main_viewmodel = GetPropEntity(player, "m_hViewModel");
		main_viewmodel.ValidateScriptScope();
		local VM_Scope = main_viewmodel.GetScriptScope();
		if ( !("GTFW_cleanup" in VM_Scope) ) VM_Scope.GTFW_cleanup <- [];
		
		if ( WepC == "tf_wearable_demoshield" ) {
			local main_viewmodel = GetPropEntity(player, "m_hViewModel");
			player.CreateCustomWearable("tf_wearable_vm", viewModel, null, main_viewmodel)
		}
		else if ( viewModel != "models/empty.mdl" ) {
			local hWearable = player.CreateCustomWearable("tf_wearable_vm", viewModel, null, weapon)
		//saves it to array in main_viewmodel scope for easy cleanup
		// meant for CreateCustomClassArms()
			VM_Scope.GTFW_cleanup.append(hWearable);
		}
		
		
	// Purpose: Overrides thirdperson model if it's a passive item, to save on ents
	
		SetPropInt(weapon, "m_nModelIndexOverrides", 0 );
		
		if ( SearchPassiveItems.find(WepC) ) {
			if ( type( worldModel ) == "string" ) {
				PrecacheModel(worldModel);
				worldModel = GetModelIndex(worldModel);
			}
			SetPropInt(weapon, "m_nModelIndexOverrides", worldModel);
		}
		else if ( worldModel != "models/empty.mdl" ) {
			local hWearable = player.CreateCustomWearable("tf_wearable", worldModel, null, weapon);
		//make wearable only show up when switching to weapon
			SetPropEntity(hWearable, "m_hWeaponAssociatedWith", weapon);
		//for custom weapons, this makes the weapon it's replacing invisible.
			DoEntFire("!self", "Clearparent", "", 0, null, weapon);
			DoEntFire("!self", "DisableShadow", "", 0, null, weapon);
		//saves it to array in main_viewmodel scope for easy cleanup
		// meant for CreateCustomClassArms()
			VM_Scope.GTFW_cleanup.append(hWearable);
		}
	}
	
	return weapon
}


//-----------------------------------------------------------------------------
// Purpose: Updates viewmodel think script.
// For fixing tf_wearable being invisible when taunting, and for custom hands on custom weapons + unintended class weapons
//-----------------------------------------------------------------------------

::GTFW.InThirdPerson <- function(player)
{
	if ( GetPropInt(player,"m_nForceTauntCam")
	 //|| player.IsTaunting()	//made a separate check, don't enable
	 //|| IsPlayerABot(player) 
	 || ( IsInMedievalMode() && Convars.GetInt( "tf_medieval_thirdperson" ) ) )
		 return true;
	return false;
}

::GTFW.AddThinkToViewModel <- function(player, GivenWeapon=null)
{
	if ( !CVAR_USE_VIEWMODEL_FIX ) {
		GTFW.DevPrint(0,"AddThinkToViewModel", "CVAR_USE_VIEWMODEL_FIX set. Ignoring function.");
		return;
	}
	
	GTFW.DevPrint(0,"AddThinkToViewModel", "Function started.");
	
	player.ValidateScriptScope();
	local playerscope = player.GetScriptScope();
	
// For changing class animations
	GTFW.WriteTableToWeaponScope_EntireLoadout(player);
	local wepScope;
	local playermodel, player_original_modelpath;
	
	if ( GivenWeapon ) {
		wepScope = GivenWeapon.GetScriptScope();
	
		if ( wepScope.gw_props & (FixClassAnims|FixClassAnimsRef|FixClassAnimsITEM) ) {
			if ( "GTFW_playermodel" in playerscope 
			  && playerscope["GTFW_playermodel"].IsValid() ) {
				playermodel = playerscope["GTFW_playermodel"];
			}
			else { 
				playermodel = GTFW.CreateTFClassModel(player);
			}
			player_original_modelpath = playerscope["GTFW_original_player_model"][player.GetPlayerClass()];
			playermodel.SetModelSimple(player_original_modelpath);
			SetPropInt(player, "m_nRenderMode", 2);
			
			if ( !("GTFW_rendermode" in playerscope) ) playerscope["GTFW_rendermode"] <- false;
		}
	}

// Think script itself.
// Two main functions for this are: 1) toggling tf_wearable visibility while in ThirdPerson and 2) Fixing class arms in first person
// It checks for whenever you go into third person or first person, based on:
// if you just taunted, or if you just used SetForceTauntCam(), or just switched weapons
// When those are found it runs functions accordingly.
// Yeah, It is kind of a mess to read and reference for your own script, sorry about that.

	local wepC, CUSTOM, last_active_building, player_tfclass;
	local wep, current_weapon;
			
	local ply_just_taunted = false;
	local ply_just_processed_thirdperson = false;
	local ply_just_processed_weaponswitch = true;
	
	const THINK_VMFIX_DELAY = 0.0;
	
	local main_viewmodel = GetPropEntity(player, "m_hViewModel");
	main_viewmodel.ValidateScriptScope()
	local VM_Scope = main_viewmodel.GetScriptScope();
	local entscriptname = "THINK_VM_FIX";
	VM_Scope[entscriptname] <- function()
	{
		current_weapon = player.GetActiveWeapon();
		
		if ( !current_weapon || !current_weapon.IsValid() ) return 0.1;
		
		local playerIsTaunting = player.IsTaunting();
		local playerIsInThirdPerson = GTFW.InThirdPerson(player);
		local playerJustSwitchedWeapons = current_weapon != wep;
	
	
	// two gates
	// if you're taunting, allow access through the gate to run the functions necessary for the player to refresh wearables visibility
	// otherwise if you hadn't switched weapons or not in TP, lockout the thinkscript (aka return to beginning of think)
		if ( playerIsTaunting || ply_just_taunted ) {
			if ( !ply_just_processed_weaponswitch && !ply_just_taunted ) {
	//			printl("taunt TRUE");
				ply_just_taunted = true;
				ply_just_processed_weaponswitch = false;
			}
	//		printl("taunting");
		}
	// If the gate is locked, just refresh all params and see if the player has switched weapons or enters thirdperson
		else if ( !playerJustSwitchedWeapons && !playerIsInThirdPerson ) {
	//		printl("idle: reset params");
			ply_just_taunted = false;
			ply_just_processed_thirdperson = false;
			ply_just_processed_weaponswitch = false;
			return THINK_VMFIX_DELAY;
		}
		
	// FUNCTION for Thirdperson wearable Fixups
	// Basically turning "m_bInitialized" on and off fixes the tf_wearables being invisible in thirdperson
	// also another gate: If this function wasn't just run, don't run it, but also check for if the player is not taunting *and* in third person, *or* if the player is taunting.
	// If those are all true then run it once
		if ( ( !ply_just_processed_thirdperson && !ply_just_taunted && playerIsInThirdPerson )
		  || ( !ply_just_processed_thirdperson && ply_just_taunted && playerIsTaunting ) ) 
		{
	//		printl("TP MODE PROCESSED");
			 
			ply_just_processed_thirdperson = true;
			
			wepScope = current_weapon.GetScriptScope();
			
			if ( playerIsTaunting ) {
				if ( playermodel && playermodel.IsValid() 
				  && wepScope.gw_props & (FixClassAnims|FixClassAnimsRef|FixClassAnimsITEM)
				  && playerscope["GTFW_rendermode"] == true ) {
					GTFW.SetUseClassAnimations_OFF(player,playerscope, playermodel, player.GetPlayerClass());
				}
			}
			
			if ( IS_CUSTOM(current_weapon) )
			{
			// Fixes taunt props
			// Custom weapons are not parented to the player because it makes them invisible.
			// If you reparent them then it allows the modelindexoverride to take effect to properly show the taunt props
				DoEntFire("!self", "SetParent", "!activator", 0, player, current_weapon);
				DoEntFire("!self", "EnableShadow", "", 0, null, current_weapon);
			}
			
		// Goes through each child wearable on player and makes them visible if they weren't already
			local wearables_list = [];
			for ( local item = player.FirstMoveChild(); item != null ; item = item.NextMovePeer() )
			{
				 if ( startswith(item.GetClassname(),"tf_wearable") )
					wearables_list.append(item)
			}
			
		// resetting m_bInitialized seems to fix visibility bugs with tf_wearable
		// and I think resetting model scale does help somehow too? I honestly forgot!!
			foreach (item in wearables_list)
			{
				SetPropInt(item, "m_AttributeManager.m_Item.m_bInitialized", 0);
				EntFireByHandle(item, "RunScriptCode", "SetPropInt(self, \x22m_AttributeManager.m_Item.m_bInitialized\x22, 1)", 0.10, item, item);
				item.SetModelScale(item.GetModelScale(),0);
				printl(GetPropEntity(item, "m_hWeaponAssociatedWith"));
			}

		}
	// FUNCTION for Firstperson weapon visbility fixups
	// Basically just fixes class arms that were bonemerged to the player's VM , and for switching player's world model animations...
	// As far as checks go, if the player just switched weapons, always run this function. Or if it's an Engineer's toolbox, run the fixup for any time a building blueprint has been switched.
	// the second possible check is if (player is NOT taunting) + (has this part not run before?)
		else if ( ( playerJustSwitchedWeapons || ( HasProp(wep, "m_hObjectBeingBuilt") && GetPropEntity(wep, "m_hObjectBeingBuilt") != last_active_building ) )
		  || ( !playerIsTaunting && ply_just_taunted && !ply_just_processed_weaponswitch ) )
		{
		//	printl("WEP SWITCHED PROCESSED");
			ply_just_processed_weaponswitch = true;
			ply_just_taunted = false;
			
			wep = current_weapon;
			wepC = wep.GetClassname();
			wep.ValidateScriptScope();
			wepScope = wep.GetScriptScope();
			CUSTOM = IS_CUSTOM(wep);
			player_tfclass = player.GetPlayerClass();
			
		// Fixes Engineer's Toolbox by setting last building instead of last weapon
			if ( HasProp(wep, "m_hObjectBeingBuilt") )
				last_active_building = GetPropEntity(wep, "m_hObjectBeingBuilt");
		
		//Checks for custom weapon and disables only non-gunslinger and non-fists
			if ( CUSTOM && !playerIsInThirdPerson ) {
				local using_fists_as_heavy = wepC == "tf_weapon_fists" && player_tfclass == 6;
				local using_gunslinger_as_engineer = wepC == "tf_weapon_robot_arm" && player_tfclass == 9;
				
				
				DoEntFire("!self", "Clearparent", "", 0, null, wep);
				DoEntFire("!self", "DisableShadow", "", 0, null, wep);
			
			
				if ( !using_fists_as_heavy || !using_gunslinger_as_engineer )
				{
					main_viewmodel.DisableDraw();		//makes firstperson weapon invisible (as well as other's class arms from the other class)
				}
			}
		
		// grabs and updates bonemerged class arms model, if it exists
			if ( "classarms" in VM_Scope ) {
				local classarms = VM_Scope["classarms"];
				
				if ( classarms && classarms.IsValid() ) {
				// if using weapon Short Circuit, disable the class arms' visibility
					local using_ShortCircuit = wepC == "tf_weapon_mechanical_arm";
					
					if ( wep && wep.IsValid() ) {
						 if ( using_ShortCircuit )
							 classarms.DisableDraw();
						else
							classarms.EnableDraw();
					}
				}
			}
		
		// updates player class model animations + bonemerged class model, as long as gtfw settings are enabled for the weapon and model exists
			if ( playermodel && playermodel.IsValid() ) {
				local WeaponHasClassAnimsFix = wepScope.gw_props & FixClassAnims;
				local WeaponHasRefPoseFix = wepScope.gw_props & FixClassAnimsRef;
				local WeaponHasITEMFix = wepScope.gw_props & FixClassAnimsITEM;
				local activityName = player.GetSequenceActivityName(player.GetSequence());
				if ( ( WeaponHasClassAnimsFix && CUSTOM )
				  || ( WeaponHasRefPoseFix && player.GetSequence() == 0 )
				  || ( WeaponHasITEMFix && wepScope.tf_class != player_tfclass && activityName.find("ITEM", 7) ) ) {
					GTFW.SetUseClassAnimations_ON(player,playerscope, playermodel, wepScope.tf_class);
				}
				else if ( playerscope.GTFW_rendermode == true ){
					GTFW.SetUseClassAnimations_OFF(player,playerscope, playermodel, player_tfclass);
				}
			}
		}
	// return to beginning of think function
		return THINK_VMFIX_DELAY;
	}
	
	AddThinkToEnt(main_viewmodel, entscriptname);	//adds think script

// debug msg
	GTFW.DevPrint(0,"AddThinkToViewModel", "Success! Added think script to "+main_viewmodel);
}
