//-----------------------------------------------------------------------------
//Purpose: Creates and bonemerges a player class model. Meant for players changing animations while using weapons unintended for the class.
// Accepts only player's handle.
// (as far as 5.4.0 this is only used in AddThinkToViewModel() under "GiveWeapon().nut" )
//-----------------------------------------------------------------------------

::GTFW.CreateTFClassModel <- function(player)
{
	if ( !player.IsPlayer() ) {
		return GTFW.DevPrint(1,"CreateTFClassModel", "Function failed. Parameter is needs player handle.");
	}
	
	player.ValidateScriptScope()
	local playerscope = player.GetScriptScope();
	
	if ( "GTFW_playermodel" in playerscope ) {
		local playermodel = playerscope["GTFW_playermodel"];
		if ( playermodel && playermodel.IsValid() ) {
			playermodel.Kill()
		}
		delete playerscope["GTFW_playermodel"];
	}
	local players_modelpath = player.GetModelName();
	if ( !( "GTFW_original_player_model" in playerscope ) )	playerscope["GTFW_original_player_model"] <- {};
	playerscope["GTFW_original_player_model"][player.GetPlayerClass()] <- players_modelpath;
	local playermodel = player.CreateCustomWearable("tf_wearable", players_modelpath, null, player);
	playermodel.DisableDraw();
	
	GTFW.DevPrint(0,"CreateTFClassModel", "Created player model " + playermodel);

	playerscope["GTFW_playermodel"] <- playermodel;

	return playermodel;
}



::GTFW.SetUseClassAnimations_ON <- function(player, playerscope, playermodel=null, player_tfclass=0)
{
	playerscope["GTFW_rendermode"] <- true;
	
	local color = GetPropInt(player, "m_clrRender");
	SetPropInt(player, "m_clrRender", color & ~(0xFF << 24) );
	
	local modelpath = GTFW_MODEL_TFCLASSES[player_tfclass];
	player.SetCustomModelWithClassAnimations(modelpath);
	
	if (playermodel) {
		SetPropInt(playermodel, "m_nRenderMode", 0);
		SetPropInt(playermodel, "m_fEffects", 129);
		SetPropInt(playermodel, "m_clrRender", color | (0xFF << 24) );
		DoEntFire("!self", "EnableShadow", "", 0, null, playermodel);
	}
}

::GTFW.SetUseClassAnimations_OFF <- function(player, playerscope=null, playermodel=null, player_tfclass=0)
{
	if (playerscope == null) {
		player.ValidateScriptScope();
		playerscope = player.GetScriptScope();
	}
	playerscope["GTFW_rendermode"] <- false;
	
	local color = GetPropInt(player, "m_clrRender");
	SetPropInt(player, "m_clrRender", color | (0xFF << 24) );
	
	local tfclass =  player.GetPlayerClass();
	local modelpath;
	if ( "GTFW_original_player_model" in playerscope
	  && tfclass in playerscope["GTFW_original_player_model"] ) modelpath = playerscope["GTFW_original_player_model"][tfclass];
	else modelpath = "";
	player.SetCustomModelWithClassAnimations(modelpath);
	
	if (playermodel) {
		SetPropInt(playermodel, "m_nRenderMode", 10);
		SetPropInt(playermodel, "m_fEffects", 161);
		SetPropInt(playermodel, "m_clrRender", color & ~(0xFF << 24) );
		DoEntFire("!self", "DisableShadow", "", 0, null, playermodel);
	}
}
